/***************************************************************************
    file        : callbacks.c
    project     : gFnRename
    begin       : 2006.04.25
    revised     : 2006.08.04 (v0.4)
                  2007.01.12 (v0.4.1)
                  2007.02.03 (v0.4.2)
                  2007.06.28 (v0.5)
                  2010.08.04 (v0.6)
    copyright   : (C)2010 Paul Schuurmans
    email       : paul.schuurmans@home.nl
 ***************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "jpslib2.h"
#include "main.h"

extern GtkWidget *App1, *DlgHelp;
extern TGlobalStruct *Gbl;
extern GPtrArray *FileStrs;

void on_App1_show (GtkWidget *widget, gpointer user_data)
{
	GtkWidget *w;
	GtkTreeView *treeview;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkComboBox *combobox;
	char temp[128] = "";
	int nval = 0;
	AppReadWriteConfig(0);
	sprintf(Gbl->HomeDir, "%s/", getenv("HOME"));
	if(!strlen(Gbl->SrcDir))  strcpy(Gbl->SrcDir, Gbl->HomeDir);
	if(!strlen(Gbl->Bookmark))  strcpy(Gbl->Bookmark, Gbl->SrcDir);
	sprintf(Gbl->Logfile, "%s/gfnrename.log", getenv("HOME"));
	AppGetVal("cb_confirmcommit", temp, &nval, &Gbl->ConfirmCommit);
	AppGetVal("cb_logfile", temp, &nval, &Gbl->UseLogfile);
	AppGetVal("cb_show_help", temp, &nval, &Gbl->ShowHelp);
	AppGetVal("cb_show_status", temp, &nval, &Gbl->ShowStatus);
	w = lookup_widget(GTK_WIDGET(App1), "label_top");
	if(Gbl->ShowHelp)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
	w = lookup_widget(GTK_WIDGET(App1), "label_bottom");
	if(Gbl->ShowStatus)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
	// Setup for Path dropdown (pathbox) //////////////////////////////////////
	renderer = gtk_cell_renderer_text_new();
	combobox = (GtkComboBox *) lookup_widget(GTK_WIDGET(App1), "combo_pathbox");
	column = gtk_tree_view_column_new_with_attributes("Path", renderer, "text", DIR_NAME, NULL);
	AppUpdatePathbox();
	// Setup for Files listbox (filebox) //////////////////////////////////////
	treeview = (GtkTreeView *) lookup_widget(GTK_WIDGET(App1), "filebox");
	// Setup Column 1
	column = gtk_tree_view_column_new_with_attributes("Old Name", renderer, "text", OLD_NAME, NULL);
	gtk_tree_view_column_set_resizable(column, TRUE);
	gtk_tree_view_column_set_sort_column_id(column, OLD_NAME);
	gtk_tree_view_append_column(treeview, column);
	// Setup Column 2
	column = gtk_tree_view_column_new_with_attributes("New Name", renderer, "text", NEW_NAME, NULL);
	gtk_tree_view_column_set_resizable(column, TRUE);
	gtk_tree_view_column_set_sort_column_id(column, NEW_NAME);
	gtk_tree_view_append_column(treeview, column);
	AppReadDirectory();
	AppUpdateList();
}

void on_App1_destroy (GtkObject *object, gpointer user_data)
{
	AppReadWriteConfig(1);
	gtk_main_quit();
}

void on_btn_cancel_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *w;
	w = lookup_widget(GTK_WIDGET(App1), "vbox_commitdlg");
	gtk_widget_hide(w);
	w = lookup_widget(GTK_WIDGET(App1), "hbox_actioncmds");
	gtk_widget_show(w);
	w = lookup_widget(GTK_WIDGET(App1), "vbox_left");
	gtk_widget_set_sensitive(w, TRUE);
	w = lookup_widget(GTK_WIDGET(App1), "vbox_options");
	gtk_widget_set_sensitive(w, TRUE);
	AppEnableCommands();
}

void on_btn_commit_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *w;
	char temp[128];
	int nval;
	AppGetVal("cb_confirmcommit", temp, &nval, &Gbl->ConfirmCommit);
	if(Gbl->ConfirmCommit) {
		w = lookup_widget(GTK_WIDGET(App1), "hbox_actioncmds");
		gtk_widget_hide(w);
		w = lookup_widget(GTK_WIDGET(App1), "vbox_left");
		gtk_widget_set_sensitive(w, FALSE);
		w = lookup_widget(GTK_WIDGET(App1), "vbox_options");
		gtk_widget_set_sensitive(w, FALSE);
		w = lookup_widget(GTK_WIDGET(App1), "vbox_commitdlg");
		gtk_widget_show(w);
		AppSetStatusText(0, "Press [OK] to commit the changes to disk, or [Cancel] if you've changed your mind.");
	}
	else  AppCommitChanges();
}

void on_btn_dlghelp_close_clicked (GtkButton *button, gpointer user_data)
{
	gtk_widget_hide(DlgHelp);
}

void on_btn_exit_clicked (GtkButton *button, gpointer user_data)
{
	AppReadWriteConfig(1);
	gtk_main_quit();
}

void on_btn_goto_mark_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *w;
	if(!FileExists(Gbl->Bookmark)) {
		sprintf(Gbl->Bookmark, "%s/", getenv("HOME"));
		w = lookup_widget(GTK_WIDGET(App1), "entry_bookmark");
		gtk_entry_set_text(GTK_ENTRY(w), Gbl->Bookmark);
	}
	strcpy(Gbl->SrcDir, Gbl->Bookmark);
	AppUpdatePathbox();
	AppReadDirectory();
	AppUpdateList();
}

void on_btn_help_clicked (GtkButton *button, gpointer user_data)
{
	if(!G_IS_OBJECT(DlgHelp))  DlgHelp = create_DlgHelp();
	gtk_widget_show(DlgHelp);
}

void on_btn_home_clicked (GtkButton *button, gpointer user_data)
{
	sprintf(Gbl->SrcDir, "%s/", getenv("HOME"));
	AppUpdatePathbox();
	AppReadDirectory();
	AppUpdateList();
}

void on_btn_mark_dir_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *w;
	strcpy(Gbl->Bookmark, Gbl->SrcDir);
	w = lookup_widget(GTK_WIDGET(App1), "entry_bookmark");
	gtk_entry_set_text(GTK_ENTRY(w), Gbl->Bookmark);
	AppEnableCommands();
}

void on_btn_ok_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *w;
	w = lookup_widget(GTK_WIDGET(App1), "vbox_commitdlg");
	gtk_widget_hide(w);
	w = lookup_widget(GTK_WIDGET(App1), "hbox_actioncmds");
	gtk_widget_show(w);
	w = lookup_widget(GTK_WIDGET(App1), "vbox_left");
	gtk_widget_set_sensitive(w, TRUE);
	w = lookup_widget(GTK_WIDGET(App1), "vbox_options");
	gtk_widget_set_sensitive(w, TRUE);
	AppCommitChanges();
}

void on_btn_rename_clicked (GtkButton *button, gpointer user_data)
{
	GtkWidget *w;
	GtkListStore *liststore;
	GtkTreeSelection *selection;
	GtkTreeIter iter;
	GPtrArray *fstrs;
	gboolean valid, clear = FALSE, bval = FALSE;
	gchar *oldname;
	TFileInfo *fi;
	TDupInfo *di, *di2;
	char newname[1024] = "", temp[1024];
	int i, j, nval;
	AppGetVal("cb_delete_last", temp, &nval, &Gbl->DoDelChars2);
	AppGetVal("entry_delete_last", temp, &Gbl->DelChars2, &bval);
	AppGetVal("cb_delete_first", temp, &nval, &Gbl->DoDelChars);
	AppGetVal("entry_delete_first", temp, &Gbl->DelChars, &bval);
	AppGetVal("cb_insert", temp, &nval, &Gbl->DoInsert);
	AppGetVal("entry_insert", Gbl->Insert, &nval, &bval);
	AppGetVal("entry_insert_at", temp, &Gbl->InsertAt, &bval);
	AppGetVal("cb_replace", temp, &nval, &Gbl->DoReplaceChar);
	AppGetVal("entry_replacechar", Gbl->Rchar1, &nval, &bval);
	AppGetVal("entry_replacewith", Gbl->Rchar2, &nval, &bval);
	AppGetVal("cb_base", temp, &nval, &Gbl->UseBase);
	AppGetVal("entry_base", Gbl->Base, &nval, &bval);
	AppGetVal("entry_offset", temp, &Gbl->Offset, &bval);
	AppGetVal("entry_digits", temp, &Gbl->Digits, &bval);
	AppGetVal("cb_prepend", temp, &nval, &Gbl->DoPrepend);
	AppGetVal("entry_prepend_digits", temp, &Gbl->PreDigits, &bval);
	AppGetVal("entry_prepend_offset", temp, &Gbl->PreOffset, &bval);
	AppGetVal("entry_prepend_increment", temp, &Gbl->PreIncrement, &bval);
	AppGetVal("cb_prefix", temp, &nval, &Gbl->UsePrefix);
	AppGetVal("entry_prefix", Gbl->Prefix, &nval, &bval);
	AppGetVal("cb_append", temp, &nval, &Gbl->DoAppend);
	AppGetVal("entry_append", Gbl->Append, &nval, &bval);
	AppGetVal("cb_extension", temp, &nval, &Gbl->UseExtension);
	AppGetVal("entry_extension", Gbl->Extension, &nval, &bval);
	AppGetVal("cb_extension_replace", temp, &nval, &Gbl->ReplaceExtension);
	AppGetVal("rb_first_dot", temp, &nval, &Gbl->ExtFirstDot);
	AppGetVal("cb_case", temp, &nval, &Gbl->DoCase);
	AppGetVal("rb_lowercase", temp, &nval, &Gbl->Lowercase);
	Gbl->Count = Gbl->PreCount = Gbl->PreError = 0;
	if(!Gbl->DoDelChars && !Gbl->DoDelChars2 && !Gbl->DoReplaceChar && !Gbl->UsePrefix && !Gbl->UseBase && 
		!Gbl->DoAppend && !Gbl->UseExtension && !Gbl->DoPrepend && !Gbl->DoCase && !Gbl->DoInsert)  clear = TRUE;
	w = lookup_widget(GTK_WIDGET(App1), "filebox");
	liststore = (GtkListStore *) gtk_tree_view_get_model(GTK_TREE_VIEW(w));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(w));
	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(liststore), &iter);
	while(valid) {
	    gtk_tree_model_get(GTK_TREE_MODEL(liststore), &iter, OLD_NAME, &oldname, -1);
		if(gtk_tree_selection_iter_is_selected(selection, &iter)) {
			if(clear)  strcpy(newname, "");
			else {
				strcpy(newname, AppMakeNewName(temp, oldname+4));
				Gbl->Count++;
				Gbl->PreCount++;
			}
			AppSetNewName(oldname, newname);
		}
		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(liststore), &iter);
    }	
	AppUpdateList();
	fstrs = g_ptr_array_new();
	for(i = 0; i < FileStrs->len && !Gbl->PreError; i++) {
		fi = g_ptr_array_index(FileStrs, i);
		if(strlen(fi->NewName)) {
			if(!(fi->St.st_mode & S_IWUSR)) {
				Gbl->PreError |= PE_FWRITE;
				g_print("on_btn_rename_clicked() - %s is not writable.\n", fi->OldName+4);
			}
			di = g_malloc0(sizeof(TDupInfo));
			strcpy(di->FileName, fi->NewName);
			g_ptr_array_add(fstrs, di);
		}
	}
	// Check for duplicates in 'New Name' column
	for(i = 0; i < fstrs->len && !Gbl->PreError; i++) {
		di = g_ptr_array_index(fstrs, i);
		for(j = 0; j < fstrs->len; j++) {
			di2 = g_ptr_array_index(fstrs, j);
			if(i == j)  continue;
			if(!strcmp(di->FileName, di2->FileName)) {
				Gbl->PreError |= PE_DUPS;
				break;
			}
		}
	}
	// Check if any files in 'New Name' column already exists
	for(i = 0; i < fstrs->len && !Gbl->PreError; i++) {
		di = g_ptr_array_index(fstrs, i);
		sprintf(newname, "%s%s", Gbl->SrcDir, di->FileName);
		if(FileExists(newname))  Gbl->PreError |= PE_EXIST;
	}
	g_ptr_array_free(fstrs, TRUE);
	AppEnableCommands();
}

void on_btn_reread_clicked (GtkButton *button, gpointer user_data)
{
	AppReadDirectory();
	AppUpdateList();
}

void on_btn_select_all_clicked (GtkButton *button, gpointer user_data)
{
	GtkTreeSelection *selection;
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lookup_widget(GTK_WIDGET(App1), "filebox")));
	if(gtk_tree_selection_count_selected_rows(selection))  gtk_tree_selection_unselect_all(selection);
	else  gtk_tree_selection_select_all(selection);
	AppEnableCommands();
}

void on_btn_show_bookmark_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "hbox_bookmark");
	Gbl->ShowBookmark = gtk_toggle_button_get_active(togglebutton);
	if(Gbl->ShowBookmark)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
}

void on_btn_show_hidden_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	Gbl->ShowHidden = gtk_toggle_button_get_active(togglebutton);
	AppReadDirectory();
	AppUpdateList();
}

void on_btn_up_one_level_clicked (GtkButton *button, gpointer user_data)
{
	int i, len;
	if((len = strlen(Gbl->SrcDir)) > 2) {
		Gbl->SrcDir[--len] = 0;
		for(i = len - 1; i; i--) {
			if(Gbl->SrcDir[i-1] == '/') {
				Gbl->SrcDir[i] = 0;
				break;
			}
		}
		AppUpdatePathbox();
		AppReadDirectory();
		AppUpdateList();
	}
}

void on_cb_append_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "entry_append");
	gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
}

void on_cb_base_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w;
	gboolean enable = gtk_toggle_button_get_active(togglebutton);
	w = lookup_widget(GTK_WIDGET(App1), "entry_base");
	gtk_widget_set_sensitive(w, enable);
	w = lookup_widget(GTK_WIDGET(App1), "hbox_base_opts");
	if(enable)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
}

void on_cb_case_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "hbox_case_opts");
	gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
}

void on_cb_confirmcommit_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	Gbl->ConfirmCommit = gtk_toggle_button_get_active(togglebutton);
}

void on_cb_delete_first_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "hbox_delete_first_opts");
	gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
}

void on_cb_delete_last_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "hbox_delete_last_opts");
	gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
}

void on_cb_extension_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "hbox_extension_opts1");
	gboolean show, enable = gtk_toggle_button_get_active(togglebutton);
	gtk_widget_set_sensitive(w, enable);
	w = lookup_widget(GTK_WIDGET(App1), "cb_extension_replace");
	show = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
	w = lookup_widget(GTK_WIDGET(App1), "hbox_extension_opts2");
	if(enable && show)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
}

void on_cb_extension_replace_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	gboolean show = gtk_toggle_button_get_active(togglebutton);
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "cb_extension");
	if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))  show = FALSE;
	w = lookup_widget(GTK_WIDGET(App1), "hbox_extension_opts2");
	if(show)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
}

void on_cb_insert_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "hbox_insert_opts");
	gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
}

void on_cb_logfile_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	Gbl->UseLogfile = gtk_toggle_button_get_active(togglebutton);
}

void on_cb_prefix_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "entry_prefix");
	gtk_widget_set_sensitive(w, gtk_toggle_button_get_active(togglebutton));
}

void on_cb_prepend_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "hbox_prepend_opts1");
	gboolean enable = gtk_toggle_button_get_active(togglebutton);
	gtk_widget_set_sensitive(w, enable);
	w = lookup_widget(GTK_WIDGET(App1), "hbox_prepend_opts2");
	if(enable)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
}

void on_cb_replace_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "entry_replacechar");
	gboolean enable = gtk_toggle_button_get_active(togglebutton);
	gtk_widget_set_sensitive(GTK_WIDGET(w), enable);
	w = lookup_widget(GTK_WIDGET(App1), "hbox_replace_with");
	if(enable)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
}

void on_cb_show_help_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "label_top");
	Gbl->ShowHelp = gtk_toggle_button_get_active(togglebutton);
	if(Gbl->ShowHelp)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
}

void on_cb_show_status_toggled (GtkToggleButton *togglebutton, gpointer user_data)
{
	GtkWidget *w = lookup_widget(GTK_WIDGET(App1), "label_bottom");
	Gbl->ShowStatus = gtk_toggle_button_get_active(togglebutton);
	if(Gbl->ShowStatus)  gtk_widget_show(w);
	else  gtk_widget_hide(w);
}

void on_combo_pathbox_changed (GtkComboBox *combobox, gpointer user_data)
{
	gchar gtxt[1024] = "";
	if(!Gbl->InSetup) {
		strcpy(gtxt, ComboGetActiveText(combobox));
		if(strcmp(gtxt, Gbl->SrcDir)) {
			strcpy(Gbl->SrcDir, gtxt);
			AppUpdatePathbox();
			AppReadDirectory();
			AppUpdateList();
		}
	}
}

gboolean on_filebox_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	if(event->type == GDK_2BUTTON_PRESS && AppTryChangeDir())  return(TRUE);
	return(FALSE);
}

gboolean on_filebox_button_release_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	AppEnableCommands();
	return(FALSE);
}

gboolean on_filebox_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	if(event->keyval == GDK_Return && AppTryChangeDir())  return(TRUE);
	return(FALSE);
}

gboolean on_filebox_key_release_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	AppEnableCommands();
	return(FALSE);
}
