/***************************************************************************
    file        : main.c
    project     : gFnRename
    begin       : 2006.04.25
    revised     : 2006.08.04 (v0.4)
                  2007.01.12 (v0.4.1)
                  2007.02.03 (v0.4.2)
                  2007.06.28 (v0.5)
                  2010.08.04 (v0.6)
    copyright   : (C)2010 Paul Schuurmans
    email       : paul.schuurmans@home.nl
 ***************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
 
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "main.h"
#include "jpslib2.h"

GtkWidget *App1, *DlgHelp;
TGlobalStruct *Gbl;
GPtrArray *FileStrs;

char *Options[OPTIONCOUNT][2] = {
	{ "cb_delete_first", "B" },
	{ "entry_delete_first", "N" },
	{ "cb_delete_last", "B" },
	{ "entry_delete_last", "N" },
	{ "cb_insert", "B" },
	{ "entry_insert", "S" },
	{ "entry_insert_at", "N" },
	{ "cb_replace", "B" },
	{ "entry_replacechar", "S" },
	{ "entry_replacewith", "S" },
	{ "cb_base", "B" },
	{ "entry_base", "S" },
	{ "entry_offset", "N" },
	{ "entry_digits", "N" },
	{ "cb_prepend", "B" },
	{ "entry_prepend_digits", "N" },
	{ "entry_prepend_offset", "N" },
	{ "entry_prepend_increment", "N" },
	{ "cb_prefix", "B" },
	{ "entry_prefix", "S" },
	{ "cb_append", "B" },
	{ "entry_append", "S" },
	{ "cb_extension", "B" },
	{ "entry_extension", "S" },
	{ "cb_extension_replace", "B" },
	{ "rb_first_dot", "B" },
	{ "rb_last_dot", "B" },
	{ "cb_case", "B" },
	{ "rb_lowercase", "B" },
	{ "rb_uppercase", "B" },
	{ "btn_show_bookmark", "B" },
	{ "entry_bookmark", "S" },
	{ "cb_confirmcommit", "B" },
	{ "cb_logfile", "B" },
	{ "cb_show_help", "B" },
	{ "cb_show_status", "B" }
};

int AppChangeDir ()
{
	DIR *dp;
	GtkTreeView *treeview;
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
	gchar *fname;
	char tempdir[1024] = "";
	treeview = (GtkTreeView *) lookup_widget(GTK_WIDGET(App1), "filebox");
	selection = gtk_tree_view_get_selection(treeview);
	if(gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, OLD_NAME, &fname, -1);
		sprintf(tempdir, "%s%s/", Gbl->SrcDir, fname+4);
		g_free(fname);
		if(IsDirectory(tempdir) == FALSE)  return(0);
		if((dp = opendir(tempdir)) == NULL) {
			AppSetStatusText(1, "Error: AppChangeDir(), opendir(%s) failed, dp=%p", tempdir, dp);
			g_print("Error: AppChangeDir(), opendir(%s) failed, dp=%p\n", tempdir, dp);
			return(0);
		}
		else {
			closedir(dp);
			strcpy(Gbl->SrcDir, tempdir);
			AppReadDirectory();
			AppUpdateList();
		}
	}
	return(1);
}

void AppCommitChanges ()
{
	FILE *fn = NULL;
	TFileInfo *fi;
	char oldname[1024], newname[1024];
	int i, cnt = 0;
	if((cnt = AppGetRenameCount()) < 1)  return;
	Gbl->Count = Gbl->Errors = 0;
	if(Gbl->UseLogfile) {
		if((fn = fopen(Gbl->Logfile, "a+")) == NULL) {
			Gbl->UseLogfile = FALSE;
			AppSetStatusText(1, "Error: AppCommitChanges(), fopen(%s, \"a+\") failed, fn=%p", Gbl->Logfile, fn);
			g_print("Error: AppCommitChanges(), fopen(%s, \"a+\") failed, fn=%p\n", Gbl->Logfile, fn);
			Gbl->Errors++;
		}
		else  fprintf(fn, "%s\n", Gbl->SrcDir);
	}
	for(i = 0; i < FileStrs->len; i++) {
		fi = g_ptr_array_index(FileStrs, i);
		if(strlen(fi->NewName)) {
			sprintf(newname, "%s%s", Gbl->SrcDir, fi->NewName);
			sprintf(oldname, "%s%s", Gbl->SrcDir, fi->OldName+4);
			if(FileExists(newname)) {
				AppSetStatusText(1, "Error: AppCommitChanges(), FileExists(%s)", g_basename(newname));
				g_print("Error: AppCommitChanges(), FileExists(%s)\n", newname);
				if(fn != NULL)  fprintf(fn, "--Rename Error: %s already exists\n", g_basename(newname));
				Gbl->Errors++;
			}
			else {
				if((rename(oldname, newname)) < 0) {
					AppSetStatusText(1, "Error: AppCommitChanges(), rename(%s, %s) failed", g_basename(oldname), g_basename(newname));
					g_print("Error: AppCommitChanges(), rename(%s, %s) failed\n", oldname, newname);
					if(fn != NULL)  fprintf(fn, "--Rename Error: %s -> %s failed. Check permissions.\n", g_basename(oldname), g_basename(newname));
					Gbl->Errors++;
				}
				else {
					if(fn != NULL)  fprintf(fn, "  %s -> %s\n", g_basename(oldname), g_basename(newname));
					Gbl->Count++;
				}
			}
		}
	}
	if(fn != NULL) { fprintf(fn, "\n"); fclose(fn); }
	AppReadDirectory();
	AppUpdateList();
}

void AppEnableCommands ()
{
	GtkWidget *rw, *cw;
	GtkTreeSelection *selection;
	int selcnt = 0, rencnt = 0;
	rw = lookup_widget(GTK_WIDGET(App1), "btn_rename");
	cw = lookup_widget(GTK_WIDGET(App1), "btn_commit");
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(lookup_widget(GTK_WIDGET(App1), "filebox")));
	selcnt = gtk_tree_selection_count_selected_rows(selection);
	rencnt = AppGetRenameCount();
	if(Gbl->PreError > 1) {
		switch(Gbl->PreError) {
		  case PE_DUPS: AppSetStatusText(PE_DUPS, "Error: Duplicate names in the 'New Name' column."); break;
		  case PE_EXIST: AppSetStatusText(PE_EXIST, "Error: One or more files in the 'New Name' column already exists."); break;
		  case PE_WRITE: AppSetStatusText(PE_WRITE, "Error: The current directory is not writable."); break;
		  case PE_FWRITE: AppSetStatusText(PE_EXIST, "Error: One or more files in the 'Old Name' column is not writable."); break;
		  default: break;
		}
		switch(Gbl->PreError) {
		  case PE_WRITE: AppSetStatusText(0, "Change to a writable directory or change the permissions."); break;
		  case PE_FWRITE: AppSetStatusText(0, "Change the permissions of files that you want to rename, then press [Re-Read Directory]."); break;
		  default: AppSetStatusText(0, "Change the renaming options for the files that caused the error."); break;
		}
		gtk_widget_set_sensitive(rw, (selcnt && !(Gbl->PreError & PE_WRITE)) ? TRUE : FALSE);
		gtk_widget_set_sensitive(cw, FALSE);
	}
	else {
		if(selcnt)  AppSetStatusText(0, "Set the desired renaming options, and then press the [Rename] button.");
		else if(rencnt)  AppSetStatusText(0, "Press the [Commit] button to commit the changes to disk, or select other files to rename.");
		else  AppSetStatusText(0, "Select the files to rename, or choose a different directory.");
		gtk_widget_set_sensitive(rw, (selcnt && !Gbl->PreError) ? TRUE : FALSE);
		gtk_widget_set_sensitive(cw, (rencnt && !Gbl->PreError) ? TRUE : FALSE);
	}
	if(Gbl->Errors)  AppSetStatusText(1, "Encountered %d errors.  Check log file or console output.", Gbl->Errors);
	else if(!Gbl->PreError)  AppSetStatusText(1, "%d items in list, %d selected, %d ready to commit.", FileStrs->len, selcnt, rencnt);
}

int AppGetRenameCount ()
{
	GtkWidget *w;
	GtkListStore *liststore;
	GtkTreeIter iter;
	gboolean valid;
	gchar *newname = NULL;
	int cnt = 0;
	w = lookup_widget(GTK_WIDGET(App1), "filebox");
	liststore = (GtkListStore *) gtk_tree_view_get_model(GTK_TREE_VIEW(w));
	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(liststore), &iter);
	while(valid) {
		gtk_tree_model_get(GTK_TREE_MODEL(liststore), &iter, NEW_NAME, &newname, -1);
		if(strlen(newname))  cnt++;
		valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(liststore), &iter);
	}	
	if(newname)  g_free(newname);
	return(cnt);
}

int AppGetVal (char *name, char *sval, int *nval, gboolean *bval)
{
	int retval = 0;
	GtkWidget *w;
	gchar *txt = NULL;
	int i, ndx = -1, len = 0;
	for(i = 0; i < OPTIONCOUNT; i++) {
		if(!strcmp(name, Options[i][0])) {
			ndx = i;
			break;
		}
	}
	if(ndx == -1) { g_print("Option Not Found: %s\n", name); return(0); }
	if((w = lookup_widget(GTK_WIDGET(App1), name)) == NULL) { g_print("Widget Not Found: %s\n", name); return(0); }
	switch(Options[ndx][1][0]) {
	  case 'S':
		txt = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
		strcpy(sval, txt);
		retval = 1;
		break;
	  case 'N':
		txt = gtk_editable_get_chars(GTK_EDITABLE(w), 0, -1);
		len = strlen(txt);
		for(i = 0; i < len; i++) {
			switch(txt[i]) {
			  case '0': case '1': case '2': case '3': case '4': case '5': 
			  case '6': case '7': case '8': case '9':  break;
			  default:  len = 0;  break;
			}
		}
		if(len && txt[len-1] != '*')  *nval = atoi(txt);
		else  *nval = -1;
		retval = 2;
		break;
	  case 'B':
		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)))  *bval = TRUE;
		else  *bval = FALSE;
		retval = 3;
		break;
	  default:
	 	g_print("Option Type Not Supported! Check Code!");
	 	break;
	}
	g_free(txt);
	return(retval);
}

char *AppMakeNewName (char *buffer, char *name)
{
	char temp[MAXPATH], temp2[MAXPATH], dir[MAXPATH], file[MAXPATH], ext[MAXPATH];
	int len = 0, i = 0;	//, offset = 0;
	strcpy(buffer, name);
	if(Gbl->DoDelChars2) {
		FnSplit(buffer, dir, file, ext);
		FnMerge(temp2, dir, file, "");
		if((len = strlen(temp2)) > Gbl->DelChars2) {
			temp2[len-Gbl->DelChars2] = 0;
			FnSplit(temp2, dir, file, temp);
			FnMerge(temp, dir, file, ext);
		}
		else {
			AppSetStatusText(1, "Error: Number of characters to delete exceeds name length.");
			temp[0] = 0;
		}
		strcpy(buffer, temp);
	}
	if(Gbl->DoDelChars) {
		if(strlen(buffer) < Gbl->DelChars) {
			AppSetStatusText(1, "Error: Number of characters to delete exceeds name length.");
			temp[0] = 0;
		}
		else  strcpy(temp, (buffer+Gbl->DelChars));
		strcpy(buffer, temp);
	}
	if(Gbl->DoInsert) {
		strcpy(temp, buffer);
		if(strlen(temp) > Gbl->InsertAt) {
			temp[Gbl->InsertAt] = 0;
			strcat(temp, Gbl->Insert);
			strcat(temp, buffer + Gbl->InsertAt);
		}
		else {
			AppSetStatusText(1, "Error: Offset for insertion exceeds name length.");
			temp[0] = 0;
		}
		strcpy(buffer, temp);
	}
	if(Gbl->DoReplaceChar && Gbl->Rchar1) {
		strcpy(temp, buffer);
		StrReplace(temp, Gbl->Rchar1, buffer, Gbl->Rchar2);
		strcpy(buffer, temp);
	}
	if(Gbl->UseBase) {
		FnSplit(buffer, dir, file, ext);
		switch(Gbl->Digits) {
		  case 0: case 1:  sprintf(temp, "%s%d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  case 2:  sprintf(temp, "%s%02d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  case 3:  sprintf(temp, "%s%03d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  case 4:  sprintf(temp, "%s%04d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  case 5:  sprintf(temp, "%s%05d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  case 6:  sprintf(temp, "%s%06d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  case 7:  sprintf(temp, "%s%07d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  case 8:  sprintf(temp, "%s%08d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  case 9:  sprintf(temp, "%s%09d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		  default:  sprintf(temp, "%s%010d.%s", Gbl->Base, Gbl->Offset + Gbl->Count, ext);  break;
		}
		strcpy(buffer, temp);
		len = strlen(buffer) - 1;
		if(len && buffer[len] == '.')  buffer[len] = 0;
	}
	if(Gbl->DoPrepend) {
		switch(Gbl->PreDigits) {
		  case 0: case 1:  sprintf(temp, "%d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  case 2:  sprintf(temp, "%02d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  case 3:  sprintf(temp, "%03d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  case 4:  sprintf(temp, "%04d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  case 5:  sprintf(temp, "%05d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  case 6:  sprintf(temp, "%06d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  case 7:  sprintf(temp, "%07d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  case 8:  sprintf(temp, "%08d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  case 9:  sprintf(temp, "%09d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		  default:  sprintf(temp, "%010d%s", Gbl->PreOffset + (Gbl->PreCount * Gbl->PreIncrement), buffer);  break;
		}
		strcpy(buffer, temp);
	}
	if(Gbl->UsePrefix) {
		sprintf(temp, "%s%s", Gbl->Prefix, buffer);
		strcpy(buffer, temp);
	}
	if(Gbl->DoAppend) {
		FnSplit(buffer, dir, file, ext);
		sprintf(temp, "%s%s.%s", file, Gbl->Append, ext);
		strcpy(buffer, temp);
	}
	if(Gbl->UseExtension) {
		if(Gbl->ReplaceExtension) {
			if(Gbl->ExtFirstDot) {
				FnSplit(buffer, dir, file, ext);
				sprintf(temp, "%s.%s", file, Gbl->Extension);
			}
			else {
				strcpy(temp, buffer);
				len = strlen(temp);
				for(i = len; i; i--) {
					if(temp[i] == '.') {
						temp[i] = 0;
						break;
					}
				}
				sprintf(temp, "%s.%s", temp, Gbl->Extension);
			}
		}
		else  sprintf(temp, "%s.%s", buffer, Gbl->Extension);
		strcpy(buffer, temp);
	}
	if(Gbl->DoCase) {
		if(Gbl->Lowercase) {
			strcpy(temp, StrCvt(buffer, TOLOW));
			strcpy(buffer, temp);
		}
		else {
			strcpy(temp, StrCvt(buffer, TOUP));
			strcpy(buffer, temp);
		}
	}
	len = strlen(buffer) - 1;
	if(len && buffer[len] == '.')  buffer[len] = 0;
	if(!strcmp(buffer, name))  buffer[0] = 0;
	return(buffer);
}

int AppReadDirectory ()
{
	GtkWidget *w;
	gboolean enable = FALSE;
	DIR *dp;
	struct dirent *entry;
	struct stat st;
	TFileInfo *fi;
	char filename[1024];
	int cnt = 0;
	if((dp = opendir(Gbl->SrcDir)) == NULL) {
		AppSetStatusText(1, "Error: AppReadDirectory(), opendir(%s) failed.", Gbl->SrcDir);
		g_print("Error: AppReadDirectory(), opendir(%s) failed.\n", Gbl->SrcDir);
		return(0); 
	}
	ClearArray(FileStrs);
	Gbl->PreError = 0;
	while((entry = readdir(dp)) != NULL) {
		sprintf(filename, "%s%s", Gbl->SrcDir, entry->d_name);
		if(stat(filename, &st) != 0)  continue;
		if(entry->d_name[0] == '.') {
			if(Gbl->ShowHidden == FALSE)  continue;
			if(entry->d_name[1] == 0)  continue;
			if(entry->d_name[1] == '.' && entry->d_name[2] == 0)  continue;
		}
		if(S_ISREG(st.st_mode) || (S_ISDIR(st.st_mode) && strcmp(".", entry->d_name))) {
			fi = g_malloc0(sizeof(TFileInfo));
			stat(filename, &fi->St);
			sprintf(fi->OldName, "[%c] %s", (S_ISDIR(fi->St.st_mode)) ? 'D' : 'f', entry->d_name);
			g_ptr_array_add(FileStrs, fi);
			cnt++;
		}
	}
	closedir(dp);
	w = lookup_widget(GTK_WIDGET(App1), "btn_home");
	gtk_widget_set_sensitive(w, (strcmp(Gbl->SrcDir, Gbl->HomeDir)) ? TRUE : FALSE);
	enable = (strcmp(Gbl->SrcDir, Gbl->Bookmark)) ? TRUE : FALSE;
	w = lookup_widget(GTK_WIDGET(App1), "btn_goto_mark");
	gtk_widget_set_sensitive(w, enable);
	w = lookup_widget(GTK_WIDGET(App1), "btn_mark_dir");
	gtk_widget_set_sensitive(w, enable);
	w = lookup_widget(GTK_WIDGET(App1), "btn_up_one_level");
	gtk_widget_set_sensitive(w, (strlen(Gbl->SrcDir) > 1) ? TRUE : FALSE);
	if(Gbl->Errors)  AppSetStatusText(1, "Encountered %d errors.  Check log file or console output.", Gbl->Errors);
	if(access(Gbl->SrcDir, W_OK) == -1)  Gbl->PreError |= PE_WRITE;
	return(cnt);
}

void AppReadWriteConfig (int writeflag)
{
	GtkWidget *w;
	int i, nval = 0, pos = 0;
	char *sval, mode[8];
	gboolean bval = FALSE;
	FILE *fn = NULL;
	char configfile[MAXPATH] = "";
	sprintf(configfile, "%s/.gfnrename.cfg", getenv("HOME"));
	sval = g_malloc0(4096);
	strcpy(mode, writeflag ? "w+" : "rb");
	if((fn = fopen(configfile, mode)) == NULL) {
		AppSetStatusText(1, "Error: AppReadWriteConfig(%d), fopen(%s, \"%s\") failed.", writeflag, configfile, mode);
		g_print("Error: AppReadWriteConfig(%d), fopen(%s, \"%s\") failed.\n", writeflag, configfile, mode);
		g_free(sval);
		return;
	}
	for(i = 0; i < OPTIONCOUNT; i++) {
		if(writeflag) {
			AppGetVal(Options[i][0], sval, &nval, &bval);
			switch(Options[i][1][0]) {
			  case 'S': case 'T': fprintf(fn, "%s=%s\n", Options[i][0], sval); break;
			  case 'N': case 'D': fprintf(fn, "%s=%d\n", Options[i][0], nval); break;
			  case 'B': fprintf(fn, "%s=%s\n", Options[i][0], bval ? "TRUE" : "FALSE"); break;
			  default: break;
			}
		}
		else {
			switch(Options[i][1][0]) {
			  case 'S': case 'T': GetConfigString(fn, Options[i][0], sval, ""); break;
			  case 'N': case 'D': GetConfigInt(fn, Options[i][0], &nval, 0); break;
			  case 'B': GetConfigBool(fn, Options[i][0], &bval, FALSE); break;
			  default: g_print("Unknown Option: '%c'\n", Options[i][1][0]); break;
			}
			w = lookup_widget(GTK_WIDGET(App1), Options[i][0]);
			switch(Options[i][1][0]) {
			  case 'N':
				if(nval == -1)  strcpy(sval, "");
				else  sprintf(sval, "%d", nval);
			  case 'S':
				pos = 0;
				gtk_editable_delete_text(GTK_EDITABLE(w), 0, -1);
				gtk_editable_insert_text(GTK_EDITABLE(w), sval, strlen(sval), &pos);
				break;
			  case 'B':
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), bval);
				break;
			  default: g_print("Unknown Option: '%c'\n", Options[i][1][0]); break;
			}
		}
	}
	fclose(fn);
	if(!writeflag) {
		AppSetStatusText(1, "AppReadWriteConfig(%d), fopen(%s, \"rb\") successful, using config settings.", writeflag, configfile);
		w = lookup_widget(GTK_WIDGET(App1), "entry_bookmark");
		strcpy(Gbl->Bookmark, gtk_entry_get_text(GTK_ENTRY(w)));
		if(!FileExists(Gbl->Bookmark)) {
			sprintf(Gbl->Bookmark, "%s/", getenv("HOME"));
			gtk_entry_set_text(GTK_ENTRY(w), Gbl->Bookmark);
		}
	}
	g_free(sval);
}

void AppSetNewName (char *oldname, char *newname)
{
	TFileInfo *fi;
	int i;
	for(i = 0; i < FileStrs->len; i++) {
		fi = g_ptr_array_index(FileStrs, i);
		if(!strcmp(oldname, fi->OldName)) {
			strcpy(fi->NewName, newname);
			break;
		}
	}
}

void AppSetStatusText (int status, char *txt, ...)
{
	GtkWidget *w;
	char msg[1024];
	va_list args;
	va_start(args, txt);
	vsprintf(msg, txt, args);
	va_end(args);
	w = lookup_widget(GTK_WIDGET(App1), status ? "label_bottom" : "label_top");
	gtk_label_set_text(GTK_LABEL(w), msg);
	if((status > 1) && !Gbl->ShowStatus) {
		w = lookup_widget(GTK_WIDGET(App1), "cb_show_status");
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE);
	}
}

int AppTryChangeDir ()
{
	GtkTreeView *treeview;
	GtkTreeSelection *selection;
	treeview = (GtkTreeView *) lookup_widget(GTK_WIDGET(App1), "filebox");
	selection = gtk_tree_view_get_selection(treeview);
	if(gtk_tree_selection_count_selected_rows(selection) > 1)  return(0);
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
	if(AppChangeDir())  AppUpdatePathbox();
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
	return(1);
}

void AppUpdateList ()
{
	GtkTreeView *treeview;
	GtkListStore *list;
	GtkTreeIter iter;
	GtkTreeSelection *treesel;
	TFileInfo *fi;
	char *text[2];
	int i = 0;
	for(i = 0; i < 2; i++)  text[i] = g_malloc0(512);
	treeview = (GtkTreeView *) lookup_widget(GTK_WIDGET(App1), "filebox");
	list = gtk_list_store_new(F_COLUMNS, G_TYPE_STRING, G_TYPE_STRING);
	for(i = 0; i < FileStrs->len; i++) {
		gtk_list_store_append(list, &iter);
		fi = g_ptr_array_index(FileStrs, i);
		strcpy(text[0], fi->OldName);
		strcpy(text[1], fi->NewName);
		gtk_list_store_set(list, &iter, OLD_NAME, text[0], NEW_NAME, text[1], -1);
	}
	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(list), OLD_NAME, GTK_SORT_ASCENDING);
	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(list));
	treesel = gtk_tree_view_get_selection(treeview);
	gtk_tree_selection_set_mode(treesel, GTK_SELECTION_MULTIPLE);
	for(i = 0; i < 2; i++)  g_free(text[i]);
	g_object_unref(list);
	AppEnableCommands();
}

void AppUpdatePathbox ()
{
	GtkComboBox *combo;
	GtkTreeModel *model;
	char txt[MAXPATH] = "";
	int i, len, cnt;
	combo = (GtkComboBox *) lookup_widget(GTK_WIDGET(App1), "combo_pathbox");
	model = gtk_combo_box_get_model(combo);
	Gbl->InSetup = 1;
	cnt = gtk_tree_model_iter_n_children(model, NULL);
	for(i = cnt; i >= 0; i--)  gtk_combo_box_remove_text(combo, i);
	strcpy(txt, Gbl->SrcDir);
	gtk_combo_box_append_text(combo, txt);
	if((len = strlen(txt)) > 1) {
		txt[len-1] = 0;
		len = strlen(txt);
		for(i = len; i >= 0; i--) {
			if(txt[i] == '/')  gtk_combo_box_append_text(combo, txt);
			else  txt[i] = 0;
		}
	}
	gtk_combo_box_set_active(combo, 0);
	Gbl->InSetup = 0;
}
	
int main (int argc, char *argv[])
{
	gtk_set_locale();
	gtk_init(&argc, &argv);
	add_pixmap_directory(PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps");
	Gbl = g_malloc0(sizeof(TGlobalStruct));
	FileStrs = g_ptr_array_new();
	App1 = create_App1();
	DlgHelp = create_DlgHelp();
	gtk_widget_show(App1);
	gtk_main();
	g_ptr_array_free(FileStrs, TRUE);
	g_free(Gbl);
	return(0);
}
