/***************************************************************************
    file        : main.h
    project     : gFnRename
    begin       : 2006.04.25
    revised     : 2006.08.04 (v0.4)
                  2007.01.12 (v0.4.1)
                  2007.02.03 (v0.4.2)
                  2007.06.28 (v0.5)
                  2010.08.04 (v0.6)
    copyright   : (C)2010 Paul Schuurmans
    email       : paul.schuurmans@home.nl
 ***************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
 
#include <sys/stat.h>
#include <sys/timeb.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <utime.h>

#define APPNAME		"gFnRename v0.6"
#define OPTIONCOUNT	36

#define GDK_Return	0xFF0D

// PreError codes
#define PE_DUPS		2
#define PE_EXIST	4
#define PE_WRITE	8
#define PE_FWRITE	16

// Limits
#define MAXPATH		260
#define MAXDIR		260
#define MAXFILE		260
#define MAXEXT		260

#define FN_DIR		0x01
#define FN_FILE		0x02
#define FN_EXT		0x04

// Modal Result for all dialogs
#define MR_CANCEL	0
#define MR_OK		1
#define MR_YES		1
#define MR_NO		2

// Modes for FileTime
#define FT_GET		0
#define FT_SET		1

// Dir codes for StrJustify
#define JUST_LEFT   	(-1)
#define JUST_CENTER   	0
#define JUST_RIGHT    	1

// Format codes for StrCvt
#define RWHITE		1  	// Remove white space.
#define RLWHITE    	2	// Remove leading white space.
#define RTWHITE    	4	// Remove trailing white space.
#define REDUCE	   	8	// Reduce white space to 1 blank.
#define NOQUOTE   	16	// Quoted substrings not altered.
#define TOUP	  	32	// Convert to upper case.
#define TOLOW	  	64	// Convert to lower case.
#define RCONTROL 	128	// Remove all control characters.
#define TOCASE	 	256	// Toggle case

// Miscellaneous
#define BLANK	 	' '
#define NEWLINE 	'\n'
#define TAB	 	'\t'
#define NUL	 	'\0'
#define MAXSRCHLEN	64
#define BUFFERSIZE	32767

typedef struct {
	gboolean ShowHidden;
	char SrcDir[MAXPATH];
	char HomeDir[MAXPATH];
	char Bookmark[MAXPATH];
	gboolean DoDelChars, DoDelChars2;
	int DelChars, DelChars2;
	gboolean DoInsert;
	char Insert[80];
	int InsertAt;
	gboolean DoReplaceChar;
	char Rchar1[MAXPATH], Rchar2[MAXPATH];
	gboolean UsePrefix;
	char Prefix[MAXPATH];
	gboolean UseBase;
	char Base[MAXPATH];
	int Offset, Digits;
	gboolean DoAppend;
	char Append[MAXPATH];
	gboolean UseExtension, ReplaceExtension, ExtFirstDot;
	char Extension[MAXPATH];
	gboolean DoPrepend;
	int PreDigits, PreOffset, PreIncrement;
	gboolean DoCase, Lowercase;
	gboolean UseLogfile;
	char Logfile[MAXPATH];
	gboolean ConfirmCommit;
	gboolean ShowBookmark, ShowHelp, ShowStatus;
	int Count, PreCount, Errors, PreError, InSetup;
} TGlobalStruct;

typedef struct {
	char FileName[MAXPATH];
} TDupInfo;

typedef struct {
	char OldName[MAXPATH];
	char NewName[MAXPATH];
	struct stat St;
} TFileInfo;

enum {
	OLD_NAME,
	NEW_NAME,
	F_COLUMNS
};

enum {
	DIR_NAME,
	D_COLUMNS
};

///// Function Prototypes ///////////////////////////////////////////////////

int AppChangeDir (void);
void AppCommitChanges (void);
void AppEnableCommands (void);
int AppGetRenameCount (void);
int AppGetVal (char *name, char *sval, int *nval, gboolean *bval);
char *AppMakeNewName (char *buffer, char *name);
int AppReadDirectory (void);
void AppReadWriteConfig (int writeflag);
void AppSetNewName (char *oldname, char *newname);
void AppSetStatusText (int status, char *txt, ...);
int AppTryChangeDir (void);
void AppUpdateList (void);
void AppUpdatePathbox (void);
